//Geoget2 GPX Garmin

//..............................................................................
//GPX for newest Garmin GPS
//Makro vzdy predpoklada nejnovejsi testovaci verzi GG2

//..............................................................................
//Authors:
//Medwyn

//..............................................................................
{Changelog:
=== 0.1 (3. 8. 2009) ===
   * Uvodni verze
   
=== 0.2 (10. 6. 2010) ===
   * Pridana promenna EXPORT_WAYPOINTS. Pomoci promenne se nastavuje, zda makro bude exportovat dodatecne waypointy.
   * Pridana promenna EXPORT_STARTING_AS_REFERENCE. V pripade presunu kese na finalni waypoint vyexportuje puvodni startovni souradnice jako referencni bod. (Drive implicitni chovani pri presunu souradnic, dnes volitelne.)
   * Pridana moznost exportovat libovolne tagy v ramci EXPORT_INFO pomoci EXPORT_TAGS.

=== 0.3 (1. 11. 2010) ===
   * Makro bylo kompletne prelozeno do cestiny
   * Pridana moznost exportovat kese se zadanymi finalovymi souradnicemi do specialni kategorie. V GPS jsou pote zobrazeny s ikonkou zavrene truhlicky (na mape i v seznamu kesi). Nevyhodou tohoto reseni je fakt, ze takto upravene kese nelze pote filtrovat pomoci integrovanych filtru v GPS (protoze GPS kategorii nezna).    
   * Popis waypointu by mel byt nyni zobrazen i v navigaci Dakota (popis je vkladan jak do tagu <desc>, tak do tagu <cmt>)
   * Waypointy maji definovany ikonky dle typu, aby je bylo mozno jednoduse rozlisit v seznamu a v mape
   * Pridana moznost definovat si klicova slova, na jejichz vyskyt bude pote prohledavan listing. Kazde nalezene slovo bude barevne zvyrazneno a na tuto skutecnost je upozorneno na zacatku listingu. Prakticke napriklad pro zvyrazneni textu "bonus".       
   * Pridana moznost odstranovat prebytecny seznam waypointu, ktery Groundspeak vklada na konec listingu v PQ
   * Pridana moznost zobrazit si seznam kesi, ktere maji prilis dlouhy listing a pravdepodobne dojde v navigaci k jeho zkraceni.
   * Export nyni exportuje pouze MAX_WPTS pocet waypointu. V pripade, ze je tento pocet prekrocen, jiz nejsou do souboru dalsi waypointy zapisovany.   
}

{$INCLUDE gpxgarmin.config.pas}

const
  TAG_EVAL = 'Hodnoceni';
  TAG_EVALCOUNT = 'Hodnoceni-Pocet';

var
  ExportedGC,ExportedWpt : Integer;
  ContainedHighlightWord : Boolean;
  exportTags,truncatedListings: TStringList;

function ExportExtension: string;
begin
  result := 'GPX';
end;

function ExportDescription: string;
begin
  result := 'GPX pro nove Garmin GPS';
end;

{Metoda vraci otucneny text}
function Bold(text: String) : String;
begin
  Result := '<b>' + text + '</b>';
end;

{Metoda prevede string s pozadovanymi tagy oddelenymi znakem '|' do stringlistu }
function TagsToList(tags:string):TStringList;
var s:string;
begin
  Result:=TStringList.Create;
  while length(tags) > 0 do begin
    s:=Fetch(tags,'|');
    if(s<>'') then Result.Add(s);
  end;
end;

{ Metoda vypise hodnoty tagu, pokud jsou nastaveny}
function OutTags(geo: TGeo;tags:TStringList) : String;
var
  i:Integer;
  s:string;
begin
  Result:='';
  for i:=0 to tags.Count-1 do
  begin
    s:= ReplaceString(geo.TagValues(tags[i]),CRLF,', ');
    if(s<>'') then Result:=Result+Bold(tags[i] + ': ')+ s +'<br>';
  end;
  if(Result<>'') then Result:='<br>'+Result;
end;

function ExportHeader: string;
begin
  {Inicializace}
  exportTags:=TagsToList(EXPORT_TAGS);
  truncatedListings := TStringList.Create;  

  {Hlavicka}
  Result := '<?xml version="1.0" encoding="utf-8"?>' + CRLF;
  Result := Result + '<gpx xmlns:xsd="http://www.w3.org/2001/XMLSchema"'
    + ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0"'
    + ' creator="Groundspeak Pocket Query"'
    + ' xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd'
    + ' http://www.groundspeak.com/cache/1/0 http://www.groundspeak.com/cache/1/0/cache.xsd"'
    + ' xmlns="http://www.topografix.com/GPX/1/0">' + CRLF;
  Result := Result + ' <desc>Geocache file</desc>' + CRLF;
  Result := Result + ' <author>' + Geoget_Version + '</author>' + CRLF;
  Result := Result + ' <time>' + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', now) + '</time>' + CRLF;
  Result := Result + ' <keywords>cache, geocache, groundspeak</keywords>' + CRLF + CRLF;
end;

function ExportFooter: string;
begin
  result := '</gpx>' + CRLF;
end;

function ExportAfter(value: string): string;
var
   s,truncfile: String;
   Size: Integer;
begin
  Result := '';
  
  {Zpracujeme seznam oriznutych listingu}
  if (MAX_LISTING_CHARS <> '0') then
  begin
    truncfile := ReplaceString(GEOGET_SCRIPTFULLNAME,GEOGET_SCRIPTNAME,'') + 'truncated exports\';
    ForceDirectories(truncfile);
    truncfile := truncfile + RegexExtract('[^\\]*$',value) + ' - ' + FormatDateTime('yyyymmddhhnnss',now) + '.ggt';
    if (truncatedListings.count > 0) then truncatedListings.SaveToFile(truncfile);  
  end;

  If (SHOW_EXPORT_WARNINGS = '1') Then
  begin
    If (ExportedGC > StrToInt(MAX_CACHES)) Then s := s + 'Pozor! Maximalni mozny pocet kesi (' + MAX_CACHES + ') byl prekrocen (' + IntToStr(ExportedGC) + ').' + CRLF + 'V GPS pravdepodobne nebude mozno zobrazit vsechny vyexportovane kese.' + CRLF + CRLF;
    If (ExportedWpt > StrToInt(MAX_WPTS)) Then s := s + 'Pozor! Maximalni mozny pocet waypointu (' + MAX_WPTS + ') byl prekrocen (' + IntToStr(ExportedWpt) + ').' + CRLF + 'Bylo proto vyexportovano pouze prvnich ' + MAX_WPTS + ' waypointu ze zobrazeneho seznamu.' + CRLF + CRLF;
    Size := GetFileSize(value);
    If (Size > StrToInt(MAX_SIZE)) Then s := s + 'Pozor! Maximalni mozna velikost souboru (' + MAX_SIZE + ' bajtu) byla prekrocena (' + IntToStr(Size) + ' bajtu).' + CRLF + 'Je mozne, ze GPS nebude schopna exportovany soubor nacist. Pokuste se upravit nastaveni makra tak, aby velikost vysledneho souboru byla pod touto hranici.' + CRLF + CRLF;
    If ((truncatedListings.count > 0) and (MAX_LISTING_CHARS <> '0')) Then s := s + 'Pozor! ' + IntToStr(truncatedListings.count)+ ' kesi obsahuje prilis dlouhe listingy a GPS je mozna nebude schopna zobrazit cele.' + CRLF + CRLF;
    If (s <> '') Then 
    begin
      If ((truncatedListings.count > 0) and (MAX_LISTING_CHARS <> '0')) then
      begin
         s := s + 'Prejete si nyni zobrazit v okne GeoGetu seznam problematickych kesi, ktere maji prilis dlouhy listing?';
         
         if (MessBox(s,'Chyba',4) = 6) then
         begin
             //Zobrazit seznam
             GeoListClear;
             GeoListFileAdd(truncfile);
         end;
      end
      else Showmessage(s);
    end;
  end;
  
  {Uvolnime pamet}
  exportTags.Free;
  truncatedListings.Free;
end;

{Metoda vraci minimum z dvou predanych hodnot}
function Minimum(a, b : Integer) : Integer;
begin
  If a < b then Result := a
  else Result := b;
end;

{Metoda vraci obarveny text}
function Color(Text, TargetColor: String) : String;
begin
  Result := '<font color=' + TargetColor + '>' + Text + '</font>';
end;

{Metoda v listingu zvyrazni vyskyty definovanych klicovych slov}
function HighlightWords(LongDesc: String) : String;
begin
	Result := RegexReplace('(?si)\S*('+ TEXT_HIGHLIGHT +')\S*', LongDesc, '<font color=' + TEXT_COLOR_HIGHLIGHT + '><b>$0</b></font>', true);
	ContainedHighlightWord := (LongDesc <> Result);
end;

{Metoda vymaze text s additional waypointy pridavany groundspeakem na konec listingu v PQ}
function DeleteAddWptsInListing(LongDesc : String) : String;
begin
   Result := RegexReplace('<p>Additional Waypoints</p>.*$', LongDesc, '', false);
end;

{Metoda ocisti predany text o nektere HTML tagy}
function CleanupHtml(Text : String) : String;
begin
    Result := Text;
    If (CLEANUP_HTML = '1') Then
    begin
      // TAG -> ''
      //SPAN, TABLE, TD, TR, TH, DIV, FONT, A, I, IMG, MARQUEE, CENTER, BLOCKQUOTE
      Result := RegexReplace('(?si)< *?(/|)?(span|table|td|tr|th|div|font|a|i|img|marquee|center|blockquote).*?>', Result, '', false);
     
      // TAG -> Other tag
      // P, LI, UL, OL
      Result := RegexReplace('(?si)< *?(p|li|ul|ol).*?>', Result, '<p>', false);
      // /P, /LI, /UL, /OL
      Result := RegexReplace('(?si)< *?/(p|li|ul|ol).*?>', Result, '</p>', false);

      //Remove unnecessary CRLF
      Result := ReplaceString(Result,CRLF, ' ');
    end;
end;


{Metoda vraci naformatovany hint}
function FormatHint(text : String) : String;
begin
  Text := ReplaceString(Text, '<oe>', ' ');
  If (EXPORT_HINT_FORMAT = 'R') then Result := Reverse(text)
  Else If (EXPORT_HINT_FORMAT = 'C') then Result := Rot13(text)
  Else If (EXPORT_HINT_FORMAT = 'F') then Result := text
  Else Result := 'Spatne zvolena promenna EXPORT_HINT_FORMAT';
end;

{Pokud jsou po poslednim "Found it" logu i logy jineho typu, metoda vypise jejich statistiku}
function LogStat(geo: TGeo) : String;
var
   tab :TSQliteTable;
   i : Integer;
begin
     i := Geoget_DB.GetTableValue('SELECT count(key) FROM geolog WHERE id = ''' + geo.ID + ''' AND dt >= (SELECT max(dt) FROM geolog WHERE id = ''' + geo.ID + ''' AND type IN (''Found it'',''Webcam Photo Taken'',''Attended'')) AND type NOT IN (''Found it'',''Webcam Photo Taken'',''Attended'') LIMIT 1;');
     if i > 0 then
     begin
       try
        Result := Color(Bold('Od posledniho nalezu bylo na kesi nekolik podezrelych logu'),TEXT_COLOR_DISABLED) + ' (';
        tab := Geoget_DB.GetTable('SELECT type, count(key) AS cnt FROM geolog WHERE id = ''' + geo.ID + ''' AND dt >= (SELECT max(dt) FROM geolog WHERE id = ''' + geo.ID + ''' AND type IN (''Found it'',''Webcam Photo Taken'',''Attended'')) AND type NOT IN (''Found it'',''Webcam Photo Taken'',''Attended'') GROUP BY type ORDER BY cnt DESC;', false);
        try
          while not tab.EOF do
          begin
            Result := Result + tab.FieldAsString(tab.FieldIndex['cnt']) + 'x ' + tab.FieldAsString(tab.FieldIndex['type']);
            if tab.next then Result := Result + ', ';
          end;
          Result := Result + ').<br>';
        finally
          tab.free;
        end;
       finally
       end;
     end;
end;

function ExportLog(LogID,LogType,FinderID,Finder,Text :String; Date :TDateTime) : String;
var
  s : String;
begin
  Result := Result + '    <groundspeak:log id="' + LogID + '">' + CRLF;
  Result := Result + '     <groundspeak:date>' 
    + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', Date) + '</groundspeak:date>' + CRLF;
  Result := Result + '     <groundspeak:type>' + LogType + '</groundspeak:type>' + CRLF;
  s := '';
  if FinderID <> '' then
    s := ' id="' + FinderID + '"'; 
  Result := Result + '     <groundspeak:finder' + s + '>' + cdata(Finder) + '</groundspeak:finder>' + CRLF;
  Result := Result + '     <groundspeak:text encoded="False">' + cdata(CleanupHtml(Text)) + '</groundspeak:text>' + CRLF;
  Result := Result + '    </groundspeak:log>' + CRLF;
end;

{Metoda vraci text formatovany jako html s informacemi o predanem WPT}
function ExportWaypointInfo(Coords,PrefixID,WptType,Name,Description,Comment :String) : String;
begin
   Result := Bold(PrefixID + ' ' + Name) + ' (' + WptType +')<br>'
            + '  ' + ReplaceString(Coords, '', ' ') + '<br>';
            if Description <> '' then Result := Result + CleanupHtml(Description) + '<br>';
            if Comment <>     '' then Result := Result + CleanupHtml(ReplaceString(Comment,CRLF, '<br>')) + '<br>';
end;

{Metoda vraci jmeno symbolu, ktery ma byt pouzit pro waypoint}
function WaypointSymbol(WptType : String): String;
begin
  Result := '';
  case WptType of
    'Final Location':           begin Result := 'Flag, Red'; end; 
    'Parking Area':             begin Result := 'Parking Area'; end;
    'Question to Answer':       begin Result := 'Flag, Green'; end;
    'Reference Point':          begin Result := 'Navaid, Blue'; end;
    'Stages of a Multicache':   begin Result := 'Flag, Blue'; end;
    'Trailhead' :               begin Result := 'Navaid, Green'; end;
    else                        begin Result := WptType; end;
  end;  
end;

function ExportWaypoint(Lat,Lon,PrefixID,ID,WptType,Name,Description,URL,Comment :String) : String;
var
  s : String;
begin
  If (EXPORT_WAYPOINTS = '1') then
  Begin 
    ExportedWpt := ExportedWpt + 1;
    if (ExportedWpt <= StrToInt(MAX_WPTS)) then
    begin

      Result := Result + ' <wpt lat="' + Lat + '" lon="' + Lon + '">' + CRLF;
      Result := Result + '  <time>'
        + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', GC.Hidden) + '</time>' + CRLF;
      //Zmena jmena waypointu na citelnejsi
      If (RENAME_WAYPOINTS = '1') Then
      Begin
        s := '';
        //Exportovani TAGU do jmena
        If (EXPORT_NAMETAG = '1') Then
        Begin
          if (GC.haveFinal AND (EXPORT_FINAL_AS_STARTING = '1')) then     s := s + 'C';
          if GC.isOwner then           s := s + 'O';
          if GC.isFound then           s := s + 'F';
          if GC.isArchived then        s := s + 'A'
          else if GC.isDisabled then   s := s + 'D';
          if (s <> '') then s := s + ' ';
          if ((EXPORT_EVALUATION_IN_NAMETAG = '1') and (GC.TagHaveCategory(TAG_EVAL)) and (GC.TagHaveCategory(TAG_EVALCOUNT)))
             Then s := s + ReplaceString(GC.TagValues(TAG_EVAL),'%','') + '/' + ReplaceString(GC.TagValues(TAG_EVALCOUNT),'x','') + ' ';
          s := GC.IDTag + '/' + PrefixID + ' ' + s;
        end;
        s := s + UtfToAscii(GC.Name) + ' (' + UtfToAscii(Name) + ')';
      End
      Else
        s := ID;   
      Result := Result + '  <name>' + cdata(s) +'</name>' + CRLF;
    //  Result := Result + '  <cmt>' + cdata(Description) +'</cmt>' + CRLF;
      s := Name;
      if Description <> '' then s := s + '/' + CleanupHtml(Description);
      if Comment <>     '' then s := s + '/' + CleanupHtml(Comment); 
     
      Result := Result + '  <cmt>' + cdata(s) +'</cmt>' + CRLF;
      Result := Result + '  <desc>' + cdata(s) +'</desc>' + CRLF;
    
    //  Result := Result + '  <url>' + cdata(URL) +'</url>' + CRLF;
    //  Result := Result + '  <urlname>' + cdata(Name) + '</urlname>' + CRLF;
      Result := Result + '  <sym>' + WaypointSymbol(WptType) + '</sym>' + CRLF
      Result := Result + '  <type>Waypoint|' + WptType + '</type>' + CRLF;
      Result := Result + ' </wpt>' + CRLF + CRLF;
    end;
  end;
end;

function ExportPoint: string;
var
  s,lat,lon,longdesc: string;
  n: integer;
begin
  Result := '';
  ContainedHighlightWord := false;  
  //Export for Geocaches
   
  if (not GC.isEmptyCoord) and (GC.isListed) then
  begin
    ExportedGC := ExportedGC + 1;
    if (GC.haveFinal AND (EXPORT_FINAL_AS_STARTING = '1')) then
    begin
      lat := GC.CorrectedLat;
      lon := GC.CorrectedLon;
    end
    else
    begin
      lat := GC.Lat;
      lon := GC.Lon;
    end;
  
    Result := Result + ' <wpt lat="' + lat + '" lon="' + lon + '">' + CRLF;
    Result := Result + '  <time>'
      + formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', GC.Hidden) + '</time>' + CRLF;
    Result := Result + '  <name>' + GC.ID +'</name>' + CRLF;
    Result := Result + '  <desc>' + cdata(GC.Name + ' by ' + GC.Author
          + ', ' + GC.CacheType + ' (' + GC.Difficulty + '/' + GC.Terrain + ')') +'</desc>' + CRLF;
    Result := Result + '  <url>' + cdata(GC.URL) +'</url>' + CRLF;
    Result := Result + '  <urlname>' + cdata(GC.Name) + '</urlname>' + CRLF;
    if GC.IsFound then
      Result := Result + '  <sym>Geocache Found</sym>' + CRLF
    else
      Result := Result + '  <sym>Geocache</sym>' + CRLF;
    Result := Result + '  <type>Geocache|' + GC.CacheType + '</type>' + CRLF;
    if GC.IsDisabled then
      s := 'available="False"'
    else
      s := 'available="True"';
    if GC.IsArchived then
      s := s + ' archived="True"'
    else
      s := s + ' archived="False"';
    s := 'id="' + GC.CacheID +'" ' + s;
    Result := Result + '  <groundspeak:cache ' + s + ' xmlns:groundspeak="http://www.groundspeak.com/cache/1/0">' + CRLF;
    
    //Uprava LongDesc
    LongDesc := CleanupHtml(GC.LongDescription);
    
    if TEXT_HIGHLIGHT <> '' then LongDesc := HighlightWords(LongDesc);
    if CLEANUP_GROUNDSPEAK_WAYPOINTLIST = '1' then LongDesc := DeleteAddWptsInListing(LongDesc);   
    
    //Exportovani tagu do jmena
    If (EXPORT_NAMETAG = '1') Then
      Begin
        s := '';
        if (GC.haveFinal AND (EXPORT_FINAL_AS_STARTING = '1')) then     s := s + 'C';
        if GC.isOwner then           s := s+ 'O';
        if GC.isFound then           s := s+ 'F';
        if GC.isArchived then        s := s+ 'A'
        else if GC.isDisabled then   s := s+ 'D';
        if (s <> '') then s := s + ' ';
        if ((EXPORT_EVALUATION_IN_NAMETAG = '1') and (GC.TagHaveCategory(TAG_EVAL)) and (GC.TagHaveCategory(TAG_EVALCOUNT)))
           Then s := s + ReplaceString(GC.TagValues(TAG_EVAL),'%','') + '/' + ReplaceString(GC.TagValues(TAG_EVALCOUNT),'x','') + ' ';
        s := UtfToAscii(GC.IDTag + ' ' + s + GC.Name);
      End
    Else
      s := GC.Name;    
    Result := Result + '   <groundspeak:name>' + cdata(s) + '</groundspeak:name>' + CRLF;
    Result := Result + '   <groundspeak:placed_by>' + cdata(GC.Author) +'</groundspeak:placed_by>' + CRLF;
    if GC.OwnerID <> '' then
      Result := Result + '   <groundspeak:owner id="' + GC.OwnerID + '">' + cdata(GC.Author) +'</groundspeak:owner>' + CRLF;
    if (GC.haveFinal and (FINAL_CATEGORY <> '')) then s := FINAL_CATEGORY else s := GC.CacheType;   
    Result := Result + '   <groundspeak:type>' + s +'</groundspeak:type>' + CRLF;
    Result := Result + '   <groundspeak:container>' + GC.Size +'</groundspeak:container>' + CRLF;
    Result := Result + '   <groundspeak:difficulty>' + GC.Difficulty +'</groundspeak:difficulty>' + CRLF;
    Result := Result + '   <groundspeak:terrain>' + GC.Terrain +'</groundspeak:terrain>' + CRLF;
    Result := Result + '   <groundspeak:country>' + GC.Country +'</groundspeak:country>' + CRLF;
    Result := Result + '   <groundspeak:state>' + GC.State +'</groundspeak:state>' + CRLF;
    //Exportovani dodatecnych informaci
    s := '';
    If (EXPORT_INFO = '1') Then
      Begin
        if (GC.haveFinal AND (EXPORT_FINAL_AS_STARTING = '1')) then     s := s + Bold('Souradnice kese byly posunuty na misto finalky.') + '<br>';
        if GC.isOwner then         s := s + Color(Bold('Jste vlastnik teto kese!'),TEXT_COLOR_OWNED) + '<br>';
        if GC.isFound then         s := s + Color(Bold('Tuto kes jste jiz nasel (' + formatdatetime('dd". "mm". "yyyy', GC.Found) +').'),TEXT_COLOR_FOUND) + '<br>';
        if GC.isArchived then      s := s + Color(Bold('Kes je archivovana!'),TEXT_COLOR_DISABLED) + '<br>'
        else if GC.isDisabled then s := s + Color(Bold('Kes je docasne nepristupna!'),TEXT_COLOR_DISABLED) + '<br>';
        if ContainedHighlightWord then s := s + Color(Bold('V listingu byla zvyraznena definovana klicova slova!'),TEXT_COLOR_HIGHLIGHT) + '<br>';
        s := s + LogStat(GC);
        if (s <> '') then s := s + '<br>';
        s := s + 
        Bold('Zalozena: ') + formatdatetime('dd". "mm". "yyyy', GC.Hidden) + '<br>' +
        Bold('Naposledy aktualizovana: ') + formatdatetime('dd". "mm". "yyyy', GC.Updated) + '<br>' +
        Bold('Naposledy nalezena: ') + formatdatetime('dd". "mm". "yyyy', GC.LastFound) + '<br>';
        s:= s +OutTags(GC,exportTags);
        if (GC.Comment <> '') then s := s + '<br>' + Bold('Poznamka: ') + ReplaceString(GC.Comment,CRLF, '<br>') + '<br>';
        if ((EXPORT_HINT_IN_INFO = '1') And (GC.Hint <> '')) Then
          s := s + Bold('Napoveda: ') + FormatHint(GC.Hint) + '<br>'; 
        
        s := s + '<hr>' + CleanupHtml(GC.ShortDescription);
      End
    Else
      s := s + CleanupHtml(GC.ShortDescription);  
    Result := Result + '   <groundspeak:short_description html="True">' + cdata(AnsiToUtf(s)) +'</groundspeak:short_description>' + CRLF;
    
    //Exportovani popisu waypointu na konec listingu
    If ((EXPORT_WP_INFO = '1') And (GC.Waypoints.Count > 0)) Then
     Begin
        s := '<hr>';
        for n := 0 to GC.Waypoints.Count - 1 do
        begin
          if (n >0) then s := s + '<br>';
          s := s + ExportWaypointInfo(GC.Waypoints[n].getCoord,GC.Waypoints[n].PrefixID,GC.Waypoints[n].WptType,GC.Waypoints[n].Name,GC.Waypoints[n].Description,GC.Waypoints[n].Comment);
       end;
        LongDesc := LongDesc + s;
      End;
    
    //Kontrola delky listingu
    if (MAX_LISTING_CHARS <> '0') 
      then if (Length(LongDesc) > StrToInt(MAX_LISTING_CHARS)) then truncatedListings.Add(GC.ID);

    Result := Result + '   <groundspeak:long_description html="True">' + cdata(LongDesc) +'</groundspeak:long_description>' + CRLF;
    Result := Result + '   <groundspeak:encoded_hints>' + cdata(GC.Hint) +'</groundspeak:encoded_hints>' + CRLF;
    Result := Result + '   <groundspeak:logs>' + CRLF;
    for n := 0 to Minimum(GC.Logs.count - 1,StrToInt(EXPORT_LOGS_MAXCOUNT)-1) do
    begin
       Result := Result + ExportLog(GC.Logs[n].LogID,GC.Logs[n].LogType,GC.Logs[n].FinderID,GC.Logs[n].Finder,GC.Logs[n].Text,GC.Logs[n].Date);
    end;
    Result := Result + '   </groundspeak:logs>' + CRLF;
    Result := Result + '  </groundspeak:cache>' + CRLF;
    Result := Result + ' </wpt>' + CRLF + CRLF;
  end;
  //Export for Waypoints
  
  //Pokud ma keska upravene souradnice, nejprve pridame novy waypoint s odkazem na originalni vychozi souradnice
  If (GC.HaveFinal AND (EXPORT_FINAL_AS_STARTING = '1') AND (EXPORT_STARTING_AS_REFERENCE = '1')) then
  begin
    Result := Result + ExportWaypoint(GC.Lat,GC.Lon,START_COORDINATES_PREFIX,ReplaceString(GC.ID,'GC', START_COORDINATES_PREFIX),'Reference Point','Vychozi souradnice','Originalni vychozi souradnice',GC.URL,'');
  end;
  
  for n := 0 to GC.Waypoints.Count - 1 do
    if (not GC.Waypoints[n].isEmptyCoord) and (GC.Waypoints[n].isListed) then
    begin
      //Pokud exportujeme kesku na upravene souradnice, nechceme na stejnem miste jeste final waypoint/y
      If Not ((GC.HaveFinal) AND (EXPORT_FINAL_AS_STARTING = '1') AND (GC.Waypoints[n].Lat = GC.CorrectedLat) AND (GC.Waypoints[n].Lon = GC.CorrectedLon) AND (GC.Waypoints[n].WptType = 'Final Location')) then
      begin   
        //Pokud exportujeme pouze final waypointy u nalezenych kesi
        if GC.isFound and (EXPORT_ONLY_FINAL_WAYPOINTS_WHEN_FOUND = '1') then
        begin
          if GC.Waypoints[n].WptType = 'Final Location' then  Result := Result + ExportWaypoint(GC.Waypoints[n].Lat,GC.Waypoints[n].Lon,GC.Waypoints[n].PrefixID,GC.Waypoints[n].ID,GC.Waypoints[n].WptType,GC.Waypoints[n].Name,GC.Waypoints[n].Description,GC.Waypoints[n].URL,GC.Waypoints[n].Comment);
        end
        else
        begin
           Result := Result + ExportWaypoint(GC.Waypoints[n].Lat,GC.Waypoints[n].Lon,GC.Waypoints[n].PrefixID,GC.Waypoints[n].ID,GC.Waypoints[n].WptType,GC.Waypoints[n].Name,GC.Waypoints[n].Description,GC.Waypoints[n].URL,GC.Waypoints[n].Comment);
        end;
      end;
    end;
end;
